#include "jsonobj.h"

namespace json {
    bool JSON_reference::isDefined() const {
        return imaginary_chain.empty();
    }

    JSON& JSON_reference::operator*() const {
        if (!isDefined())
            throw misuse("dereferencing json reference with non-empty imaginary part");
        return *last_real;
    }

    void JSON_reference::operator=(const JSON &obj) {
        JSON* cur_last_real = last_real;
        for (const auto& ck: imaginary_chain) {
            if (ck.type == undefined_array_element) {
                if (cur_last_real->type == null_symbol)
                    *cur_last_real = JSON(array);
                if (cur_last_real->type != array)
                    throw misuse("Implicit array creation on top of neither non-null nor short-array json obj is not allowed");
                cur_last_real->asArray().resize(ck.when_array_index + 1);
                cur_last_real = &cur_last_real->asArray()[ck.when_array_index];
            } else {
                if (cur_last_real->type == null_symbol)
                    *cur_last_real = JSON(dictionary);
                if (cur_last_real->type != dictionary)
                    throw misuse("Implicit dictionary creation on top of  neither non-null nor illiterate-dict json obj is not allowed");
                cur_last_real = &(cur_last_real->asDictionary()[ck.when_dictionary_key]);
            }
        }
        *cur_last_real = obj;
    }

    JSON_reference JSON_reference::operator[](size_t index) {
        if (!imaginary_chain.empty()) {
            std::vector<ImaginaryKeyChainEValue> elongated = imaginary_chain;
            elongated.push_back({undefined_array_element, index, ""});
            return {last_real, elongated};
        }
        if (last_real->isArray() && last_real->asArray().size() > index) {
            return {&last_real->asArray()[index], {}};
        }
        return {last_real, {ImaginaryKeyChainEValue{undefined_array_element, index, ""}}};
    }

    JSON_reference JSON_reference::operator[](const std::string &key) {
        if (!imaginary_chain.empty()) {
            std::vector<ImaginaryKeyChainEValue> elongated = imaginary_chain;
            elongated.push_back({undefined_dictionary_element, 0, key});
            return {last_real, elongated};
        }
        if (last_real->isDictionary() && last_real->asDictionary().count(key) > 0) {
            return {&last_real->asDictionary()[key], {}};
        }
        return {last_real, {ImaginaryKeyChainEValue{undefined_dictionary_element, 0, key}}};
    }
}
